/* class applied to the link to 'history' on a element view when last event of history has comments */
span#content-history.highlight-history-link a {
    color: red;
    font-weight: bold;
    font-size: 125%;
    text-decoration: underline dotted;
}
